package com.regula.documentreader;

import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMapKeySetIterator;
import com.regula.documentreader.api.DocumentReader;
import com.regula.documentreader.api.ParamsCustomization;
import com.regula.documentreader.api.params.Functionality;
import com.regula.documentreader.api.params.ProcessParam;
import com.regula.documentreader.api.params.DataGroups;

import android.widget.ImageView.ScaleType;
import android.graphics.Typeface;
import android.graphics.Matrix;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

// You can read more about each these settings at 
// https://github.com/regulaforensics/DocumentReader-Android/wiki/Custom-settings

public class RegulaConfig {

  static String[] extractStringArray(ReadableArray readableArray) {
    String[] result = new String[readableArray.size()];
    for (int i = 0; i < readableArray.size(); i++) {
      result[i] = readableArray.getString(i);
    }
    return result;
  }

  static int[] extractIntArray(ReadableArray readableArray) {
    int[] result = new int[readableArray.size()];
    for (int i = 0; i < readableArray.size(); i++) {
      result[i] = readableArray.getInt(i);
    }
    return result;
  }

  static float[] extractFloatArray(ReadableArray readableArray) {
    float[] result = new float[readableArray.size()];
    for (int i = 0; i < readableArray.size(); i++) {
      result[i] = (float) readableArray.getDouble(i);
    }
    return result;
  }

  public static void setConfig(DocumentReader reader, ReadableMap opts) {
    if (opts.hasKey("customization"))
      setCustomization(reader.customization, opts.getMap("customization"));
    if (opts.hasKey("functionality"))
      setFunctionality(reader.functionality, opts.getMap("functionality"));
    if (opts.hasKey("processParams"))
      reader.processParams().fromJson(convertMapToJson(opts.getMap("processParams")).toString());
  }

  public static JSONObject getConfig(DocumentReader reader) {
    JSONObject object = new JSONObject();
    try {
      object.put("customization", getCustomization(reader.customization()));
      object.put("functionality", getFunctionality(reader.functionality()));
      object.put("processParams", getProcessParams(reader.processParams()));
    } catch (JSONException e) {
    }

    return object;
  }

  public static void setFunctionality(Functionality functionality, ReadableMap opts) {
    if (opts.hasKey("mPictureOnBoundsReady"))
      functionality.setPictureOnBoundsReady(opts.getBoolean("mPictureOnBoundsReady"));
    if (opts.hasKey("mShowTorchButton"))
      functionality.setShowTorchButton(opts.getBoolean("mShowTorchButton"));
    if (opts.hasKey("mShowCloseButton"))
      functionality.setShowCloseButton(opts.getBoolean("mShowCloseButton"));
    if (opts.hasKey("mVideoCaptureMotionControl"))
      functionality.setVideoCaptureMotionControl(opts.getBoolean("mVideoCaptureMotionControl"));
    if (opts.hasKey("mShowCaptureButton"))
      functionality.setShowCaptureButton(opts.getBoolean("mShowCaptureButton"));
    if (opts.hasKey("mShowChangeFrameButton"))
      functionality.setShowChangeFrameButton(opts.getBoolean("mShowChangeFrameButton"));
    if (opts.hasKey("mShowCaptureButtonDelayFromDetect"))
      functionality.setShowCaptureButtonDelayFromDetect(opts.getInt("mShowCaptureButtonDelayFromDetect"));
    if (opts.hasKey("mShowCaptureButtonDelayFromStart"))
      functionality.setShowCaptureButtonDelayFromStart(opts.getInt("mShowCaptureButtonDelayFromStart"));
    if (opts.hasKey("mIsOnlineMode"))
      functionality.setOnlineMode(opts.getBoolean("mIsOnlineMode"));
    if (opts.hasKey("mDatabaseAutoupdate"))
      functionality.setDatabaseAutoupdate(opts.getBoolean("mDatabaseAutoupdate"));
    if (opts.hasKey("mShowSkipNextPageButton"))
      functionality.setShowSkipNextPageButton(opts.getBoolean("mShowSkipNextPageButton"));
    if (opts.hasKey("mUseAuthenticator"))
      functionality.setUseAuthenticator(opts.getBoolean("mUseAuthenticator"));
    if (opts.hasKey("mSkipFocusingFrames"))
      functionality.setSkipFocusingFrames(opts.getBoolean("mSkipFocusingFrames"));
    if (opts.hasKey("mShowCameraSwitchBtn"))
      functionality.setShowCameraSwitchBtn(opts.getBoolean("mShowCameraSwitchBtn"));
    if (opts.hasKey("mCameraFrame"))
      functionality.setCameraFrame(opts.getString("mCameraFrame"));
    if (opts.hasKey("mServiceURL"))
      functionality.setServiceURL(opts.getString("mServiceURL"));
    if (opts.hasKey("mDatabaseIdentifier"))
      functionality.setDatabaseIdentifier(opts.getString("mDatabaseIdentifier"));
    if (opts.hasKey("mBtDeviceName"))
      functionality.setBtDeviceName(opts.getString("mBtDeviceName"));
    if (opts.hasKey("mOrientation"))
      functionality.setOrientation(opts.getInt("mOrientation"));
  }

  public static void setCustomization(ParamsCustomization customization, ReadableMap opts) {
    if (opts.hasKey("mStatus"))
      customization.setStatus(opts.getString("mStatus"));
    if (opts.hasKey("mResultStatus"))
      customization.setResultStatus(opts.getString("mResultStatus"));
    if (opts.hasKey("mCameraFrameDefaultColor"))
      customization.setCameraFrameDefaultColor(opts.getString("mCameraFrameDefaultColor"));
    if (opts.hasKey("mCameraFrameActiveColor"))
      customization.setCameraFrameActiveColor(opts.getString("mCameraFrameActiveColor"));
    if (opts.hasKey("mStatusTextColor"))
      customization.setStatusTextColor(opts.getString("mStatusTextColor"));
    if (opts.hasKey("mResultStatusTextColor"))
      customization.setResultStatusTextColor(opts.getString("mResultStatusTextColor"));
    if (opts.hasKey("mResultStatusBackgroundColor"))
      customization.setResultStatusBackgroundColor(opts.getString("mResultStatusBackgroundColor"));
    if (opts.hasKey("mMultipageButtonBackgroundColor"))
      customization.setMultipageButtonBackgroundColor(opts.getString("mMultipageButtonBackgroundColor"));
    if (opts.hasKey("mTintColor"))
      customization.setTintColor(opts.getString("mTintColor"));
    if (opts.hasKey("mActivityIndicatorColor"))
      customization.setActivityIndicatorColor(opts.getString("mActivityIndicatorColor"));
    if (opts.hasKey("mShowStatusMessages"))
      customization.setShowStatusMessages(opts.getBoolean("mShowStatusMessages"));
    if (opts.hasKey("mShowResultStatusMessages"))
      customization.setShowResultStatusMessages(opts.getBoolean("mShowResultStatusMessages"));
    if (opts.hasKey("mShowHelpAnimation"))
      customization.setShowHelpAnimation(opts.getBoolean("mShowHelpAnimation"));
    if (opts.hasKey("mShowNextPageAnimation"))
      customization.setShowNextPageAnimation(opts.getBoolean("mShowNextPageAnimation"));
    if (opts.hasKey("mShowBackgroundMask"))
      customization.setShowBackgroundMask(opts.getBoolean("mShowBackgroundMask"));
    if (opts.hasKey("mCameraFrameBorderWidth"))
      customization.setCameraFrameBorderWidth(opts.getInt("mCameraFrameBorderWidth"));
    if (opts.hasKey("mStatusTextSize"))
      customization.setStatusTextSize(opts.getInt("mStatusTextSize"));
    if (opts.hasKey("mCameraFrameLineLength"))
      customization.setCameraFrameLineLength(opts.getInt("mCameraFrameLineLength"));
    if (opts.hasKey("mCameraFrameShapeType"))
      customization.setCameraFrameShapeType(opts.getInt("mCameraFrameShapeType"));
    if (opts.hasKey("mResultStatusTextSize"))
      customization.setResultStatusTextSize(opts.getInt("mResultStatusTextSize"));
    if (opts.hasKey("mHelpAnimationImageID"))
      customization.setHelpAnimationImageID(opts.getInt("mHelpAnimationImageID"));
    if (opts.hasKey("mMultipageAnimationFrontImage"))
      customization.setMultipageAnimationFrontImage(opts.getInt("mMultipageAnimationFrontImage"));
    if (opts.hasKey("mMultipageAnimationBackImage"))
      customization.setMultipageAnimationBackImage(opts.getInt("mMultipageAnimationBackImage"));
    if (opts.hasKey("mBorderBackgroundImage"))
      customization.setBorderBackgroundImage(opts.getInt("mBorderBackgroundImage"));
    if (opts.hasKey("mStatusPositionMultiplier"))
      customization.setStatusPositionMultiplier(opts.getDouble("mStatusPositionMultiplier"));
    if (opts.hasKey("mResultStatusPositionMultiplier"))
      customization.setResultStatusPositionMultiplier(opts.getDouble("mResultStatusPositionMultiplier"));
    if (opts.hasKey("mBackgroundMaskAlpha"))
      customization.setBackgroundMaskAlpha((float) opts.getDouble("mBackgroundMaskAlpha"));
    if (opts.hasKey("mStatusTextFont"))
      customization
          .setStatusTextFont(Typeface.create(opts.getString("mStatusTextFont"), opts.getInt("mStatusTextFontStyle")));
    if (opts.hasKey("mResultStatusTextFont"))
      customization.setResultStatusTextFont(
          Typeface.create(opts.getString("mResultStatusTextFont"), opts.getInt("mResultStatusTextFontStyle")));
    if (opts.hasKey("mHelpAnimationImageScaleType"))
      customization.setHelpAnimationImageScaleType(ScaleType.valueOf(opts.getString("mHelpAnimationImageScaleType")));
    if (opts.hasKey("mMultipageAnimationFrontImageScaleType"))
      customization.setMultipageAnimationFrontImageScaleType(
          ScaleType.valueOf(opts.getString("mMultipageAnimationFrontImageScaleType")));
    if (opts.hasKey("mMultipageAnimationBackImageScaleType"))
      customization.setMultipageAnimationBackImageScaleType(
          ScaleType.valueOf(opts.getString("mMultipageAnimationBackImageScaleType")));
    if (opts.hasKey("mHelpAnimationImageMatrix")) {
      Matrix matrix = new Matrix();
      matrix.setValues(extractFloatArray(opts.getArray("mHelpAnimationImageMatrix")));
      customization.setHelpAnimationImageMatrix(matrix);
    }
    if (opts.hasKey("mMultipageAnimationFrontImageMatrix")) {
      Matrix matrix = new Matrix();
      matrix.setValues(extractFloatArray(opts.getArray("mMultipageAnimationFrontImageMatrix")));
      customization.setMultipageAnimationFrontImageMatrix(matrix);
    }
    if (opts.hasKey("mMultipageAnimationBackImageMatrix")) {
      Matrix matrix = new Matrix();
      matrix.setValues(extractFloatArray(opts.getArray("mMultipageAnimationBackImageMatrix")));
      customization.setMultipageAnimationBackImageMatrix(matrix);
    }
  }

  public static JSONObject getFunctionality(Functionality functionality) {
    JSONObject object = new JSONObject();
    try {
      object.put("mPictureOnBoundsReady", functionality.isPictureOnBoundsReady());
      object.put("mShowTorchButton", functionality.isShowTorchButton());
      object.put("mShowCloseButton", functionality.isShowCloseButton());
      object.put("mVideoCaptureMotionControl", functionality.isVideoCaptureMotionControl());
      object.put("mShowCaptureButton", functionality.isShowCaptureButton());
      object.put("mShowChangeFrameButton", functionality.isShowChangeFrameButton());
      object.put("mShowCaptureButtonDelayFromDetect", functionality.getShowCaptureButtonDelayFromDetect());
      object.put("mShowCaptureButtonDelayFromStart", functionality.getShowCaptureButtonDelayFromStart());
      object.put("mIsOnlineMode", functionality.isOnlineMode());
      object.put("mDatabaseAutoupdate", functionality.isDatabaseAutoupdate());
      object.put("mShowSkipNextPageButton", functionality.isShowSkipNextPageButton());
      object.put("mUseAuthenticator", functionality.isUseAuthenticator());
      object.put("mSkipFocusingFrames", functionality.isSkipFocusingFrames());
      object.put("mShowCameraSwitchBtn", functionality.isShowCameraSwitchBtn());
      object.put("mCameraFrame", functionality.getCameraFrame());
      object.put("mServiceURL", functionality.getServiceURL());
      object.put("mDatabaseIdentifier", functionality.getDatabaseIdentifier());
      object.put("mBtDeviceName", functionality.getBtDeviceName());
      object.put("mOrientation", functionality.getOrientation());
    } catch (JSONException e) {
    }

    return object;
  }

  public static JSONObject getCustomization(ParamsCustomization customization) {
    JSONObject object = new JSONObject();
    try {
      object.put("mStatus", customization.getStatus());
      object.put("mResultStatus", customization.getResultStatus());
      object.put("mCameraFrameDefaultColor", customization.getCameraFrameDefaultColor());
      object.put("mCameraFrameActiveColor", customization.getCameraFrameActiveColor());
      object.put("mStatusTextColor", customization.getStatusTextColor());
      object.put("mResultStatusTextColor", customization.getResultStatusTextColor());
      object.put("mResultStatusBackgroundColor", customization.getResultStatusBackgroundColor());
      object.put("mMultipageButtonBackgroundColor", customization.getMultipageButtonBackgroundColor());
      object.put("mTintColor", customization.getTintColor());
      object.put("mActivityIndicatorColor", customization.getActivityIndicatorColor());
      object.put("mShowStatusMessages", customization.isShowStatusMessages());
      object.put("mShowResultStatusMessages", customization.isShowResultStatusMessages());
      object.put("mShowHelpAnimation", customization.isShowHelpAnimation());
      object.put("mShowNextPageAnimation", customization.isShowNextPageAnimation());
      object.put("mShowBackgroundMask", customization.isShowBackgroundMask());
      object.put("mCameraFrameBorderWidth", customization.getCameraFrameBorderWidth());
      object.put("mStatusTextSize", customization.getStatusTextSize());
      object.put("mCameraFrameLineLength", customization.getCameraFrameLineLength());
      object.put("mCameraFrameShapeType", customization.getCameraFrameShapeType());
      object.put("mResultStatusTextSize", customization.getResultStatusTextSize());
      object.put("mHelpAnimationImageID", customization.getHelpAnimationImageID());
      object.put("mMultipageAnimationFrontImage", customization.getMultipageAnimationFrontImage());
      object.put("mMultipageAnimationBackImage", customization.getMultipageAnimationBackImage());
      object.put("mBorderBackgroundImage", customization.getBorderBackgroundImage());
      object.put("mHelpAnimationImageScaleType", customization.getHelpAnimationImageScaleType());
      object.put("mMultipageAnimationFrontImageScaleType", customization.getMultipageAnimationFrontImageScaleType());
      object.put("mMultipageAnimationBackImageScaleType", customization.getMultipageAnimationBackImageScaleType());
      object.put("mHelpAnimationImageMatrix", customization.getHelpAnimationImageMatrix());
      object.put("mMultipageAnimationFrontImageMatrix", customization.getMultipageAnimationFrontImageMatrix());
      object.put("mMultipageAnimationBackImageMatrix", customization.getMultipageAnimationBackImageMatrix());
      object.put("mStatusTextFont", customization.getStatusTextFont());
      object.put("mResultStatusTextFont", customization.getResultStatusTextFont());
      object.put("mStatusPositionMultiplier", customization.getStatusPositionMultiplier());
      object.put("mResultStatusPositionMultiplier", customization.getResultStatusPositionMultiplier());
      object.put("mBackgroundMaskAlpha", customization.getBackgroundMaskAlpha());
    } catch (JSONException e) {
    }

    return object;
  }

  public static JSONObject getProcessParams(ProcessParam processParams) {
    JSONObject object = new JSONObject();
    try {
      object.put("scenario", processParams.scenario);
      object.put("measureSystem", processParams.measureSystem);
      object.put("uvTorchEnabled", processParams.uvTorchEnabled);
      object.put("logs", processParams.logs);
      object.put("debugSaveImages", true);
      object.put("debugSaveLogs", processParams.debugSaveLogs);
      object.put("multipageProcessing", processParams.multipageProcessing);
      object.put("dateFormat", processParams.dateFormat);
      object.put("debugSaveCroppedImages", processParams.debugSaveCroppedImages);
      object.put("sessionLogFolder", processParams.sessionLogFolder);
      object.put("disableFocusingCheck", processParams.disableFocusingCheck);
      object.put("captureButtonScenario", processParams.captureButtonScenario);
      object.put("returnUncroppedImage", processParams.returnUncroppedImage);
      if (processParams.customParams != null) {
        object.put("customParams", processParams.customParams.toString());
      } else {
        object.put("customParams", new JSONArray());
      }

      JSONArray jdocumentIDList = new JSONArray();
      if (processParams.documentIDList != null && processParams.documentIDList.length > 0) {
        int index = 0;
        for (int item : processParams.documentIDList) {
          jdocumentIDList.put(index, item);
          index++;
        }
      }
      object.put("documentIDList", jdocumentIDList);

      JSONArray jdoBarcodes = new JSONArray();
      if (processParams.doBarcodes != null && processParams.doBarcodes.length > 0) {
        int index = 0;
        for (String item : processParams.doBarcodes) {
          jdoBarcodes.put(index, item);
          index++;
        }
      }
      object.put("doBarcodes", jdoBarcodes);

      JSONArray jfieldTypesFilter = new JSONArray();
      if (processParams.fieldTypesFilter != null && processParams.fieldTypesFilter.length > 0) {
        int index = 0;
        for (int item : processParams.fieldTypesFilter) {
          jfieldTypesFilter.put(index, item);
          index++;
        }
      }
      object.put("fieldTypesFilter", jfieldTypesFilter);
    } catch (Exception e) {
    }
    return object;
  }

  public static void setRfidScenario(DocumentReader reader, ReadableMap opts) {
    if (opts.hasKey("mPaceStaticBinding"))
      reader.Instance().rfidScenario().setPaceStaticBinding(opts.getBoolean("PACE_StaticBinding"));
    if (opts.hasKey("mSignManagementAction"))
      reader.Instance().rfidScenario().setSignManagementAction(opts.getInt("mSignManagementAction"));
    if (opts.hasKey("mReadingBuffer"))
      reader.Instance().rfidScenario().setReadingBuffer(opts.getInt("mReadingBuffer"));
    if (opts.hasKey("mOnlineTAToSignDataType"))
      reader.Instance().rfidScenario().setOnlineTAToSignDataType(opts.getInt("mOnlineTAToSignDataType"));
    if (opts.hasKey("mOnlineTA"))
      reader.Instance().rfidScenario().setOnlineTA(opts.getBoolean("mOnlineTA"));
    if (opts.hasKey("mWriteEid"))
      reader.Instance().rfidScenario().setWriteEid(opts.getBoolean("mWriteEid"));
    if (opts.hasKey("mProfilerType"))
      reader.Instance().rfidScenario().setProfilerType(opts.getInt("mProfilerType"));
    if (opts.hasKey("mAuthProcType"))
      reader.Instance().rfidScenario().setAuthProcType(opts.getInt("mAuthProcType"));
    if (opts.hasKey("mBaseSMProcedure"))
      reader.Instance().rfidScenario().setBaseSMProcedure(opts.getInt("mBaseSMProcedure"));
    if (opts.hasKey("mPacePasswordType"))
      reader.Instance().rfidScenario().setPacePasswordType(opts.getInt("mPacePasswordType"));
    if (opts.hasKey("mTerminalType"))
      reader.Instance().rfidScenario().setTerminalType(opts.getInt("mTerminalType"));
    if (opts.hasKey("mUniversalAccessRights"))
      reader.Instance().rfidScenario().setUniversalAccessRights(opts.getBoolean("mUniversalAccessRights"));
    if (opts.hasKey("mAuthorizedRestrictedIdentification"))
      reader.Instance().rfidScenario()
          .setAuthorizedRestrictedIdentification(opts.getBoolean("mAuthorizedRestrictedIdentification"));
    if (opts.hasKey("mAuxVerificationCommunityID"))
      reader.Instance().rfidScenario().setAuxVerificationCommunityID(opts.getBoolean("mAuxVerificationCommunityID"));
    if (opts.hasKey("mAuxVerificationDateOfBirth"))
      reader.Instance().rfidScenario().setAuxVerificationDateOfBirth(opts.getBoolean("mAuxVerificationDateOfBirth"));
    if (opts.hasKey("mSkipAA"))
      reader.Instance().rfidScenario().setSkipAA(opts.getBoolean("mSkipAA"));
    if (opts.hasKey("mStrictProcessing"))
      reader.Instance().rfidScenario().setStrictProcessing(opts.getBoolean("mStrictProcessing"));
    if (opts.hasKey("mPkdDSCertPriority"))
      reader.Instance().rfidScenario().setPkdDSCertPriority(opts.getBoolean("mPkdDSCertPriority"));
    if (opts.hasKey("mPkdUseExternalCSCA"))
      reader.Instance().rfidScenario().setPkdUseExternalCSCA(opts.getBoolean("mPkdUseExternalCSCA"));
    if (opts.hasKey("mTrustedPKD"))
      reader.Instance().rfidScenario().setTrustedPKD(opts.getBoolean("mTrustedPKD"));
    if (opts.hasKey("mPassiveAuth"))
      reader.Instance().rfidScenario().setPassiveAuth(opts.getBoolean("mPassiveAuth"));
    if (opts.hasKey("mPassword"))
      reader.Instance().rfidScenario().setPassword(opts.getString("mPassword"));
    if (opts.hasKey("mUseSFI"))
      reader.Instance().rfidScenario().setUseSFI(opts.getBoolean("mUseSFI"));
    if (opts.hasKey("mPkdPA"))
      reader.Instance().rfidScenario().setPkdPA(opts.getString("mPkdPA"));
    if (opts.hasKey("mPkdEAC"))
      reader.Instance().rfidScenario().setPkdEAC(opts.getString("mPkdEAC"));
    if (opts.hasKey("mReadEPassport"))
      reader.Instance().rfidScenario().setReadEPassport(opts.getBoolean("mReadEPassport"));
    if (opts.hasKey("mReadEID"))
      reader.Instance().rfidScenario().setReadEID(opts.getBoolean("mReadEID"));
    if (opts.hasKey("mReadEDL"))
      reader.Instance().rfidScenario().setReadEDL(opts.getBoolean("mReadEDL"));
    if (opts.hasKey("mMrz"))
      reader.Instance().rfidScenario().setMrz(opts.getString("mMrz"));
    if (opts.hasKey("mESignPINDefault"))
      reader.Instance().rfidScenario().seteSignPINDefault(opts.getString("mESignPINDefault"));
    if (opts.hasKey("mESignPINNewValue"))
      reader.Instance().rfidScenario().seteSignPINNewValue(opts.getString("mESignPINNewValue"));
    if (opts.hasKey("mAuthorizedSTSignature"))
      reader.Instance().rfidScenario().setAuthorizedSTSignature(opts.getBoolean("mAuthorizedSTSignature"));
    if (opts.hasKey("mAuthorizedSTQSignature"))
      reader.Instance().rfidScenario().setAuthorizedSTQSignature(opts.getBoolean("mAuthorizedSTQSignature"));
    if (opts.hasKey("mAuthorizedWriteDG17"))
      reader.Instance().rfidScenario().setAuthorizedWriteDG17(opts.getBoolean("mAuthorizedWriteDG17"));
    if (opts.hasKey("mAuthorizedWriteDG18"))
      reader.Instance().rfidScenario().setAuthorizedWriteDG18(opts.getBoolean("mAuthorizedWriteDG18"));
    if (opts.hasKey("mAuthorizedWriteDG19"))
      reader.Instance().rfidScenario().setAuthorizedWriteDG19(opts.getBoolean("mAuthorizedWriteDG19"));
    if (opts.hasKey("mAuthorizedWriteDG20"))
      reader.Instance().rfidScenario().setAuthorizedWriteDG20(opts.getBoolean("mAuthorizedWriteDG20"));
    if (opts.hasKey("mAuthorizedWriteDG21"))
      reader.Instance().rfidScenario().setAuthorizedWriteDG21(opts.getBoolean("mAuthorizedWriteDG21"));
    if (opts.hasKey("mAuthorizedVerifyAge"))
      reader.Instance().rfidScenario().setAuthorizedVerifyAge(opts.getBoolean("mAuthorizedVerifyAge"));
    if (opts.hasKey("mAuthorizedVerifyCommunityID"))
      reader.Instance().rfidScenario()
          .setAuthorizedVerifyCommunityID(opts.getBoolean("mAuthorizedVerifyCommunityID"));
    if (opts.hasKey("mAuthorizedPrivilegedTerminal"))
      reader.Instance().rfidScenario()
          .setAuthorizedPrivilegedTerminal(opts.getBoolean("mAuthorizedPrivilegedTerminal"));
    if (opts.hasKey("mAuthorizedCANAllowed"))
      reader.Instance().rfidScenario().setAuthorizedCANAllowed(opts.getBoolean("mAuthorizedCANAllowed"));
    if (opts.hasKey("mAuthorizedPINManagement"))
      reader.Instance().rfidScenario().setAuthorizedPINManagment(opts.getBoolean("mAuthorizedPINManagement"));
    if (opts.hasKey("mAuthorizedInstallCert"))
      reader.Instance().rfidScenario().setAuthorizedInstallCert(opts.getBoolean("mAuthorizedInstallCert"));
    if (opts.hasKey("mAuthorizedInstallQCert"))
      reader.Instance().rfidScenario().setAuthorizedInstallQCert(opts.getBoolean("mAuthorizedInstallQCert"));
    if (opts.hasKey("ePassportDataGroups"))
      setDataGroups(reader.Instance().rfidScenario().ePassportDataGroups(), opts.getMap("ePassportDataGroups"));
    if (opts.hasKey("eIDDataGroups"))
      setDataGroups(reader.Instance().rfidScenario().eIDDataGroups(), opts.getMap("eIDDataGroups"));
    if (opts.hasKey("eDLDataGroups"))
      setDataGroups(reader.Instance().rfidScenario().eDLDataGroups(), opts.getMap("eDLDataGroups"));
  }

  private static void setDataGroups(DataGroups dataGroup, ReadableMap opts) {
    if (opts.hasKey("DG1"))
      dataGroup.setDG1(opts.getBoolean("DG1"));
    if (opts.hasKey("DG2"))
      dataGroup.setDG2(opts.getBoolean("DG2"));
    if (opts.hasKey("DG3"))
      dataGroup.setDG3(opts.getBoolean("DG3"));
    if (opts.hasKey("DG4"))
      dataGroup.setDG4(opts.getBoolean("DG4"));
    if (opts.hasKey("DG5"))
      dataGroup.setDG5(opts.getBoolean("DG5"));
    if (opts.hasKey("DG6"))
      dataGroup.setDG6(opts.getBoolean("DG6"));
    if (opts.hasKey("DG7"))
      dataGroup.setDG7(opts.getBoolean("DG7"));
    if (opts.hasKey("DG8"))
      dataGroup.setDG8(opts.getBoolean("DG8"));
    if (opts.hasKey("DG9"))
      dataGroup.setDG9(opts.getBoolean("DG9"));
    if (opts.hasKey("DG10"))
      dataGroup.setDG10(opts.getBoolean("DG10"));
    if (opts.hasKey("DG11"))
      dataGroup.setDG11(opts.getBoolean("DG11"));
    if (opts.hasKey("DG12"))
      dataGroup.setDG12(opts.getBoolean("DG12"));
    if (opts.hasKey("DG13"))
      dataGroup.setDG13(opts.getBoolean("DG13"));
    if (opts.hasKey("DG14"))
      dataGroup.setDG14(opts.getBoolean("DG14"));
    if (opts.hasKey("DG15"))
      dataGroup.setDG15(opts.getBoolean("DG15"));
    if (opts.hasKey("DG16"))
      dataGroup.setDG16(opts.getBoolean("DG16"));
  }

  private static JSONObject convertMapToJson(ReadableMap readableMap) {
    JSONObject object = new JSONObject();
    try {
      ReadableMapKeySetIterator iterator = readableMap.keySetIterator();
      while (iterator.hasNextKey()) {
        String key = iterator.nextKey();
        switch (readableMap.getType(key)) {
        case Null:
          object.put(key, JSONObject.NULL);
          break;
        case Boolean:
          object.put(key, readableMap.getBoolean(key));
          break;
        case Number:
          object.put(key, readableMap.getDouble(key));
          break;
        case String:
          object.put(key, readableMap.getString(key));
          break;
        case Map:
          object.put(key, convertMapToJson(readableMap.getMap(key)));
          break;
        case Array:
          object.put(key, convertArrayToJson(readableMap.getArray(key)));
          break;
        }
      }
    } catch (JSONException e) {
    }

    return object;
  }

  private static JSONArray convertArrayToJson(ReadableArray readableArray) {
    JSONArray array = new JSONArray();
    try {
      for (int i = 0; i < readableArray.size(); i++) {
        switch (readableArray.getType(i)) {
        case Null:
          break;
        case Boolean:
          array.put(readableArray.getBoolean(i));
          break;
        case Number:
          array.put(readableArray.getDouble(i));
          break;
        case String:
          array.put(readableArray.getString(i));
          break;
        case Map:
          array.put(convertMapToJson(readableArray.getMap(i)));
          break;
        case Array:
          array.put(convertArrayToJson(readableArray.getArray(i)));
          break;
        }
      }
    } catch (JSONException e) {
    }

    return array;
  }
}